import XCTest

import SwiftDoc

final class PathTests: XCTestCase {
    func testEmptyBaseURL() {
        XCTAssertEqual(path(for: "Class", with: ""), "Class")

        XCTAssertEqual(path(for: "(lhs:rhs:)", with: ""), "(lhs:rhs:)")
    }

    func testRootDirectoryBaseURL() {
        XCTAssertEqual(path(for: "Class", with: "/"), "/Class")

        XCTAssertEqual(path(for: "(lhs:rhs:)", with: "/"), "/(lhs:rhs:)")
    }

    func testCurrentDirectoryBaseURL() {
        XCTAssertEqual(path(for: "Class", with: "./"), "./Class")

        XCTAssertEqual(path(for: "(lhs:rhs:)", with: "./"), "./(lhs:rhs:)")
    }

    func testNestedSubdirectoryBaseURL() {
        XCTAssertEqual(path(for: "Class", with: "/path/to/directory"), "/path/to/directory/Class")
        XCTAssertEqual(path(for: "Class", with: "/path/to/directory/"), "/path/to/directory/Class")

        XCTAssertEqual(path(for: "(lhs:rhs:)", with: "/path/to/directory"), "/path/to/directory/(lhs:rhs:)")
        XCTAssertEqual(path(for: "(lhs:rhs:)", with: "/path/to/directory/"), "/path/to/directory/(lhs:rhs:)")
    }

    func testDomainBaseURL() {
        XCTAssertEqual(path(for: "Class", with: "https://example.com"), "https://example.com/Class")
        XCTAssertEqual(path(for: "Class", with: "https://example.com/"), "https://example.com/Class")

        XCTAssertEqual(path(for: "(lhs:rhs:)", with: "https://example.com"), "https://example.com/(lhs:rhs:)")
        XCTAssertEqual(path(for: "(lhs:rhs:)", with: "https://example.com/"), "https://example.com/(lhs:rhs:)")
    }

    func testDomainSubdirectoryBaseURL() {
        XCTAssertEqual(path(for: "Class", with: "https://example.com/docs"), "https://example.com/docs/Class")
        XCTAssertEqual(path(for: "Class", with: "https://example.com/docs/"), "https://example.com/docs/Class")

        XCTAssertEqual(path(for: "(lhs:rhs:)", with: "https://example.com/docs"), "https://example.com/docs/(lhs:rhs:)")
        XCTAssertEqual(path(for: "(lhs:rhs:)", with: "https://example.com/docs/"), "https://example.com/docs/(lhs:rhs:)")
    }
}
